@@ -197,13 +197,6 @@ static NCNN_FORCEINLINE __m256 _mm256_comp_fnmadd_ps(__m256 _a, const __m256 _b,
 }
 #endif
 
-#if __AVX2__
-static NCNN_FORCEINLINE __m256 loadfp16(const unsigned short* ptr)
-{
-    return _mm256_cvtph_ps(_mm_lddqu_si128((__m128i*)(ptr)));
-}
-#endif
-
 static NCNN_FORCEINLINE __m256 _mm256_fmadd_1_ps(__m256 a, __m256 b, float c)
 {
     return _mm256_comp_fmadd_ps(b, _mm256_set1_ps(c), a);
